package cn.chnmuseum.party.web.controller;

import cn.chnmuseum.party.common.log.MethodLog;
import cn.chnmuseum.party.common.log.OperModule;
import cn.chnmuseum.party.common.log.OperType;
import cn.chnmuseum.party.common.util.ListUtil;
import cn.chnmuseum.party.common.util.TimeUtils;
import cn.chnmuseum.party.common.validator.groups.Add;
import cn.chnmuseum.party.common.validator.groups.Update;
import cn.chnmuseum.party.common.vo.GenericPageParam;
import cn.chnmuseum.party.model.*;
import cn.chnmuseum.party.service.*;
import cn.chnmuseum.party.web.controller.base.BaseController;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresAuthentication;
import org.hibernate.validator.constraints.Length;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotBlank;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <pre>
 *  前端控制器
 * </pre>
 *
 * @author Danny Lee
 * @since 2021-03-26
 */
@Slf4j
@RestController
@RequestMapping("/learningProject")
@Api(tags = {"学习项目操作接口"})
public class LearningProjectController extends BaseController {

    @Resource
    private LearningProjectService learningProjectService;
    @Resource
    private LearningContentService learningContentService;
    @Autowired
    private LearningContentController learningContentController;
    @Resource
    private LearningContentBoardService learningContentBoardService;
    @Resource
    private LearningContentBoardCatService learningContentBoardCatService;
    @Resource
    private LearningContentCopyrightOwnerService learningContentCopyrightOwnerService;
    @Resource
    private CopyrightOwnerService copyrightOwnerService;
    @Resource
    private ExhibitionBoardCatService exhibitionBoardCatService;
    @Resource
    private ExhibitionBoardService exhibitionBoardService;


    @PostMapping("/save")
    @RequiresAuthentication  //@RequiresPermissions("learning:project:save")
    @ApiOperation(value = "添加", notes = "添加")
    @MethodLog(operModule = OperModule.LEARNPROJECT, operType = OperType.ADD)
    @Transactional
    public Map<String, Object> saveLearningProject(@Validated(value = {Add.class}) LearningProject learningProject) {
        LambdaQueryWrapper<LearningProject> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(LearningProject::getName, learningProject.getName());
        LearningProject one = learningProjectService.getOne(lambdaQuery);
        if (one != null) {
            return getFailResult("学习项目名称已存在！");
        }
        // 保存业务节点信息
        boolean result = learningProjectService.save(learningProject);
        if (!result) {
            return getFailResult();
        }

        // 该学习项目下的主学习内容
        LearningContent majorLearningContent = LearningContent.builder()
                .learningProjectId(learningProject.getId())
                .applicableScope("ALL_PLAT")
                .name(learningProject.getMajor())
                .cover(learningProject.getCover())
                .exhibitionBoardCatIdList(learningProject.getExhibitionBoardCatIdList())
                .copyrightOwnerIdList(learningProject.getCopyrightOwnerIdList())
                .exhibitionBoardIdList(learningProject.getExhibitionBoardIdList())
                .build();

        Map<String, Object> resultMap = learningContentController.saveLearningContent(majorLearningContent, true);
        if (!resultMap.get("resultCode").equals("200")) {
            return getFailResult();
        }
        // 返回操作结果
        if (result) {
            return getSuccessResult();
        } else {
            // 保存失败
            return getFailResult();
        }
    }

    @PutMapping("/update")
    @RequiresAuthentication  //@RequiresPermissions("learning:project:update")
    @ApiOperation(value = "修改信息", notes = "修改信息")
    @MethodLog(operModule = OperModule.LEARNPROJECT, operType = OperType.UPDATE)
    @Transactional
    public Map<String, Object> updateLearningProject(@Validated(value = {Update.class}) LearningProject learningProject) {
        // 是否重名
        LambdaQueryWrapper<LearningProject> lambdaQuery1 = Wrappers.<LearningProject>lambdaQuery();
        lambdaQuery1.eq(LearningProject::getName, learningProject.getName()).ne(LearningProject::getId, learningProject.getId());
        LearningProject one1 = learningProjectService.getOne(lambdaQuery1);
        if (one1 != null) {
            return getFailResult("学习项目名已存在！");
        }

        StringBuilder resultMsg = new StringBuilder();
        // 查询学习项目原有信息
        LearningProject project = learningProjectService.getById(learningProject.getId());
        // 项目基本信息被修改
        if (!project.getName().equals(learningProject.getName()) || !project.getRemarks().equals(learningProject.getRemarks())) {
            boolean flag = learningProjectService.updateById(learningProject);
            if (!flag) {
                return getFailResult();
            }
            resultMsg.append("项目信息修改成功,");
        } else {
            resultMsg.append("未修改项目信息,");
        }

        // 查询项目的主学习内容
        LambdaQueryWrapper<LearningContent> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(LearningContent::getLearningProjectId, learningProject.getId());
        queryWrapper.eq(LearningContent::getIsMajor, true);
        LearningContent content = this.learningContentService.getOne(queryWrapper);
        Map<String, Object> map = this.learningContentController.getById(content.getId());
        LearningContent one = (LearningContent) map.get("data");
        if (one == null) {
            return getFailResult("该学习项目下没有主学习内容");
        }
        // 判断是否修改主学习内容
        String learningProjectMajorName = learningProject.getMajor();
        String cover = learningProject.getCover();
        List<String> copyrightOwnerIdList = learningProject.getCopyrightOwnerIdList();
        List<String> exhibitionBoardCatIdList = learningProject.getExhibitionBoardCatIdList();
        List<String> exhibitionBoardIdList = learningProject.getExhibitionBoardIdList();
        if (one.getName().equals(learningProjectMajorName)
                && one.getCover().equals(cover)
                && ListUtil.compareValue(copyrightOwnerIdList, one.getCopyrightOwnerIdList())
                && ListUtil.compareValue(exhibitionBoardCatIdList, one.getExhibitionBoardCatIdList())
                && ListUtil.compareValue(exhibitionBoardIdList, one.getExhibitionBoardIdList())) {
            resultMsg.append("未修改项目主学习内容信息。");
            return getSuccessResult(resultMsg.toString());
        }

        // 查询该学习项目的子学习内容版权方、展板分类、展板
        LambdaQueryWrapper<LearningContent> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(LearningContent::getLearningProjectId, learningProject.getId());
        lambdaQueryWrapper.eq(LearningContent::getIsMajor, false);
        lambdaQueryWrapper.select(LearningContent::getId);
        List<String> list = this.learningContentService.listObjs(lambdaQueryWrapper, Object::toString);
        if (CollectionUtil.isNotEmpty(list)) {
            // 查询子学习内容的版权方
            LambdaQueryWrapper<LearningContentCopyrightOwner> query = Wrappers.lambdaQuery();
            query.in(LearningContentCopyrightOwner::getLearningContentId, list);
            query.select(LearningContentCopyrightOwner::getCopyrightOwnerId);
            List<String> list1 = this.learningContentCopyrightOwnerService.listObjs(query, Object::toString);
            // 修改主内容的版权信息中删除了子学习内容的版权信息
            if (CollectionUtil.isNotEmpty(list1) && !learningProject.getCopyrightOwnerIdList().containsAll(list1)) {
                list1.removeAll(learningProject.getCopyrightOwnerIdList());
                // 查询子学习内容中被删除的版权方信息
                List<CopyrightOwner> copyrightOwners = this.copyrightOwnerService.listByIds(list1);
                StringBuilder stringBuilder = new StringBuilder();
                copyrightOwners.forEach(s -> {
                    stringBuilder.append(s.getName());
                    if (copyrightOwners.get(copyrightOwners.size() - 1).equals(s)) {
                        stringBuilder.append("，");
                    } else {
                        stringBuilder.append("、");
                    }
                });
                return getFailResult("展板版权方：" + stringBuilder + "被子学习内容使用，不能被删除，更新失败！");
            }

            // 查询子学习内容的展板分类
            LambdaQueryWrapper<LearningContentBoardCat> query1 = Wrappers.lambdaQuery();
            query1.in(LearningContentBoardCat::getLearningContentId, list);
            query1.select(LearningContentBoardCat::getExhibitionBoardCatId);
            List<String> list2 = this.learningContentBoardCatService.listObjs(query1, Object::toString);
            // 主学习内容展板分类不能完全子学习内容的展板分类
            if (CollectionUtil.isNotEmpty(list2) && !learningProject.getExhibitionBoardCatIdList().containsAll(list2)) {
                // 差集
                list2.removeAll(learningProject.getExhibitionBoardCatIdList());
                // 查询被删除的展板分类信息
                List<ExhibitionBoardCat> learningContentBoardCats = this.exhibitionBoardCatService.listByIds(list2);
                StringBuilder stringBuilder = new StringBuilder();
                learningContentBoardCats.forEach(s -> {
                    stringBuilder.append(s.getName());
                    if (learningContentBoardCats.get(learningContentBoardCats.size() - 1).equals(s)) {
                        stringBuilder.append("，");
                    } else {
                        stringBuilder.append("、");
                    }
                });
                return getFailResult("展板类别：" + stringBuilder + "被子学习内容使用，不能被删除，更新失败");
            }

            // 查询子学习内容的展板
            LambdaQueryWrapper<LearningContentBoard> query2 = Wrappers.lambdaQuery();
            query2.in(LearningContentBoard::getLearningContentId, list);
            query2.select(LearningContentBoard::getExhibitionBoardId);
            List<String> list3 = this.learningContentBoardService.listObjs(query2, Object::toString);
            // 修改主要学习内容时删除了子学习内容所使用展板
            if (CollectionUtil.isNotEmpty(list3) && !learningProject.getExhibitionBoardIdList().containsAll(list3)) {
                // 差集
                list3.removeAll(learningProject.getExhibitionBoardIdList());
                // 查询被删除的展板信息
                LambdaQueryWrapper<ExhibitionBoard> lambdaQuery = Wrappers.lambdaQuery();
                lambdaQuery.eq(ExhibitionBoard::getPublished, true);
                lambdaQuery.in(ExhibitionBoard::getId, list3);
                List<ExhibitionBoard> exhibitionBoards = this.exhibitionBoardService.list(lambdaQuery);
                if (CollectionUtil.isNotEmpty(exhibitionBoards)) {
                    // 被删除的展板名称
                    StringBuilder stringBuilder = new StringBuilder();
                    exhibitionBoards.forEach(s -> {
                        stringBuilder.append(s.getName());
                        if (exhibitionBoards.get(exhibitionBoards.size() - 1).equals(s)) {
                            stringBuilder.append("，");
                        } else {
                            stringBuilder.append("、");
                        }
                    });
                    return getFailResult("展板：" + stringBuilder + "被子学习内容使用，不能被删除，更新失败！");
                    }
                }
        }

        LearningContent learningContent = LearningContent.builder()
                .id(one.getId())
                .name(learningProject.getMajor())
                .copyrightOwnerIdList(learningProject.getCopyrightOwnerIdList())
                .cover(learningProject.getCover())
                .exhibitionBoardCatIdList(learningProject.getExhibitionBoardCatIdList())
                .exhibitionBoardIdList(learningProject.getExhibitionBoardIdList()).build();
        Map<String, Object> resultMap = this.learningContentController.updateLearningContent(learningContent);
        resultMsg.append("修改学习内容已提交，待审核！");
        if (resultMap.get("resultCode").equals("200")) {
            resultMap.replace("message", resultMsg);
        }
        return map;
    }

    @GetMapping("/getList")
    @RequiresAuthentication  //@RequiresPermissions("learning:project:list")
    @ApiOperation(value = "获取全部列表(无分页)", notes = "获取全部列表(无分页)")
    @MethodLog(operModule = OperModule.LEARNPROJECT, operType = OperType.SELECT)
    public Map<String, Object> getLearningProjectList() {
        List<LearningProject> learningProjectList = learningProjectService.list(Wrappers.<LearningProject>lambdaQuery().orderByDesc(LearningProject::getCreateTime));
        return getResult(learningProjectList);
    }

    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "_index", value = "分页起始偏移量", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(name = "_size", value = "返回条数", paramType = "query", dataType = "Integer"),
            @ApiImplicitParam(name = "nameOrCode", value = "名称或编码", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startDate", value = "创建时间-开始", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endDate", value = "创建时间-结束", paramType = "query", dataType = "String")
    })
    @PostMapping("/getPageList")
    @RequiresAuthentication  //@RequiresPermissions("learning:project:page")
    @ApiOperation(value = "获取分页列表", notes = "获取分页列表")
    @MethodLog(operModule = OperModule.LEARNPROJECT, operType = OperType.SELECT)
    public Map<String, Object> getLearningProjectPageList(GenericPageParam genericPageParam) {
        LambdaQueryWrapper<LearningProject> queryWrapper = new LambdaQueryWrapper<>();
        // 对名称或编码模糊查询
        if (StringUtils.isNotBlank(genericPageParam.getNameOrCode())) {
            queryWrapper.like(LearningProject::getName, genericPageParam.getNameOrCode());
        }
        // 根据创建时间区间检索
        if (genericPageParam.getStartDate() != null && genericPageParam.getEndDate() != null) {
            queryWrapper.ge(LearningProject::getCreateTime, genericPageParam.getStartDate().atTime(0, 0, 0))
                    .le(LearningProject::getCreateTime, genericPageParam.getEndDate().atTime(23, 59, 59));
        }
        // 设置排序规则
        queryWrapper.orderByDesc(LearningProject::getCreateTime);
        // 设置查询内容
        queryWrapper.select(
                LearningProject::getId,
                LearningProject::getName,
                LearningProject::getRemarks,
                LearningProject::getCreateTime,
                LearningProject::getUpdateTime);
        Page<LearningProject> page = this.learningProjectService.page(getPage(), queryWrapper);

        for (LearningProject learningProject : page.getRecords()) {
            LambdaQueryWrapper<LearningContent> lambdaQueryWrapper = Wrappers.<LearningContent>lambdaQuery()
                    .eq(LearningContent::getLearningProjectId, learningProject.getId());
            List<LearningContent> learningContentList = this.learningContentService.list(lambdaQueryWrapper);
            List<String> learningContentNameList = new ArrayList<>();
            learningContentList.forEach(s -> {
                if (s.getIsMajor()) {
                    learningProject.setAuditStatus(s.getAuditStatus());
                } else {
                    learningContentNameList.add(s.getName());
                }
            });
            learningProject.setLearningContentNames(String.join("、", learningContentNameList));
        }
        return getResult(page);
    }

    @ApiOperation(value = "获取详情", notes = "获取详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "标识ID", dataType = "String", paramType = "path", required = true)
    })
    @GetMapping("/get/{id}")
    @RequiresAuthentication  //@RequiresPermissions("learning:project:get:id")
    @MethodLog(operModule = OperModule.LEARNPROJECT, operType = OperType.SELECT)
    public Map<String, Object> getById(@PathVariable("id") String id) {
        LearningProject learningProject = learningProjectService.getById(id);
        // 该项目的主学习内容
        LambdaQueryWrapper<LearningContent> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(LearningContent::getLearningProjectId, id);
        lambdaQuery.eq(LearningContent::getIsMajor, true);
        LearningContent majorLearningContent = this.learningContentService.getOne(lambdaQuery);
        if (majorLearningContent != null) {
            Map<String, Object> map = this.learningContentController.getById(majorLearningContent.getId());
            LearningContent data = (LearningContent) map.get("data");
            learningProject.setMajorLearning(data);
        }
        return getResult(learningProject);
    }

    @DeleteMapping("/delete/{id}")
    @RequiresAuthentication  //@RequiresPermissions("learning:project:delete")
    @ApiOperation(value = "根据ID删除学习项目", notes = "根据ID删除学习项目")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "id", value = "标识ID", paramType = "path", dataType = "String", required = true)
    })
    @Transactional
    @MethodLog(operModule = OperModule.LEARNPROJECT, operType = OperType.DELETE)
    public Map<String, Object> deleteLearningProject(@PathVariable("id") String id) {
        // 项目下的学习内容id
        LambdaQueryWrapper<LearningContent> queryWrapper = Wrappers.<LearningContent>lambdaQuery();
        queryWrapper.eq(LearningContent::getLearningProjectId, id);
        queryWrapper.select(LearningContent::getId);
        List<String> list = this.learningContentService.listObjs(queryWrapper, Object::toString);
        // 删除学习内容和展板关联关系
        this.learningContentBoardService.remove(Wrappers.<LearningContentBoard>lambdaUpdate().in(LearningContentBoard::getLearningContentId, list));
        // 删除学习内容和展板分类的关联关系
        this.learningContentBoardCatService.remove(Wrappers.<LearningContentBoardCat>lambdaUpdate().in(LearningContentBoardCat::getLearningContentId, list));
        // 删除学习内容和展板版权方的关联关系
        this.learningContentCopyrightOwnerService.remove(Wrappers.<LearningContentCopyrightOwner>lambdaUpdate().in(LearningContentCopyrightOwner::getLearningContentId, list));
        // 删除学习内容
        this.learningContentService.remove(Wrappers.<LearningContent>lambdaUpdate().eq(LearningContent::getLearningProjectId, id));
        // 删除学习项目
        this.learningProjectService.removeById(id);
        return getSuccessResult();
    }

}

